/*
   Schwerpunktwage
   Christoph Meier 2018
   cfd.koelleteam.de
   nach https://github.com/olkal/CG_scale
   Kalibrierung der Wiegezellen mit Examples/Calibrate/Calibrate.ino aus HX711_ADC libary -> https://github.com/olkal/HX711_ADC can be installed from the library manager
   
   ####### LCD Connection with Arduino #######
    RS => digital pin 8
    RW => GND
    E  => digital pin 9 
    D0-D3 => NC(Not Connected) 
    D4 => digital pin 10 
    D5 => digital pin 11
    D6 => digital pin 12 
    D7 => digital pin 13 
*/ 

#include <LiquidCrystal.h>
// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(8, 9, 10, 11, 12, 13);

#include <HX711_ADC.h> 
//HX711 constructor (dout pin, sck pint):
HX711_ADC LoadCell_1(5, 4); //HX711 pins front sensor (DOUT, PD_SCK)
HX711_ADC LoadCell_2(7, 6); //HX711 pins front sensor (DOUT, PD_SCK)

//Whre is the switch connected to:
int SwPin = A1;

// 
long t1;
long t2;
byte type=0;
const int AbfrageIntervall = 500; // LCD/Serial refresh interval
int changetype=0;

//*** configuration:
//*** set dimensional calibration values:
const long WingScaleDist[2] = {1090, 1342}; //calibration value in 1/10mm, projected distance between wing support points, measure with calliper
const long LEstopperDist[2] = {90, 240}; //calibration value 1/10mm, projected distance from front wing support point to leading edge (stopper pin), measure with calliper
//*** set scale calibration values (best to have the battery connected when doing calibration):
const float ldcell_1_calfactor[2] = {958, 1019.5}; // user set calibration factor load cell front (float)
const float ldcell_2_calfactor[2] = {933, 982.0}; // user set calibration factor load cell front (float)
//***
const long stabilisingtime = 3000; // tare precision can be improved by adding a few seconds of stabilising time


void setup() {
 
  
  Serial.begin(9600);

  Serial.println("CG Test");
  lcd.begin(16, 2);
  lcd.print("koelleteam.de");
  pinMode(SwPin, INPUT); 
  LoadCell_1.begin();
  LoadCell_2.begin();
  calibrate();
  delay(1000);
  Serial.println(digitalRead(SwPin));

  lcd.clear();

}

void tara(){
 LoadCell_1.tareNoDelay();
 LoadCell_2.tareNoDelay();
}

void calibrate()
{
  byte loadcell_1_rdy = 0;
  byte loadcell_2_rdy = 0;
  while ((loadcell_1_rdy + loadcell_2_rdy) < 2) { //run startup, stabilisation and tare, both modules simultaneously
    if (!loadcell_1_rdy) loadcell_1_rdy = LoadCell_1.startMultiple(stabilisingtime);
    if (!loadcell_2_rdy) loadcell_2_rdy = LoadCell_2.startMultiple(stabilisingtime);
  }
  
   LoadCell_1.setCalFactor(ldcell_1_calfactor[type]); // set calibration factor
  LoadCell_2.setCalFactor(ldcell_2_calfactor[type]); // set calibration factor
  
}

void type_switch(){
  if(digitalRead(SwPin)==HIGH){
    t2 = millis();
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("tara / mode select");
    lcd.setCursor(0, 1);
        lcd.print(">");
    lcd.setCursor(4, 1);
    if(type==0){
        lcd.print("F3X(2x2Kg)");
      }
      else{
        lcd.print("F5D(2x1Kg)");
      }   
    while(digitalRead(SwPin)==HIGH){
      if(millis()-t2 > 4000)
      {
        changetype=1;
        if(type==0){ lcd.setCursor(0, 1);lcd.print("F3F ACTIVE!2x2Kg"); }
        else{ lcd.setCursor(0, 1); lcd.print("F5D ACTIVE!2x1Kg"); }
      }
      else if(millis()-t2 > 3000)
      {
        lcd.setCursor(3, 1);
        lcd.print(">");
      }
      else if(millis()-t2 > 2000)
      {
        lcd.setCursor(2, 1);
        lcd.print(">");
      }
      else if(millis()-t2 > 1000)
      {
        lcd.setCursor(1, 1);
        lcd.print(">");
      }
      
    }
    if (changetype==1){  
      if(type==0){ type=1; }
      else{ type=0; }
      calibrate();
      changetype=0;
    }
    tara();
    lcd.clear();
  }
}
void loop() {
  //library function update() should be called at least as often as HX711 sample rate; >10Hz@10SPS, >80Hz@80SPS
  //longer delay in scetch will reduce effective sample rate (be careful with delay() in loop)
  LoadCell_1.update();
  LoadCell_2.update();

  // calculate CG and update serial/LCD
  if (t1 < millis()) {
    t1 = millis() + AbfrageIntervall;
    lcd.clear();
    float a = LoadCell_1.getData();
    float b = LoadCell_2.getData();
    
    long weightAvr[2];
    long CG;
    weightAvr[0] = a * 100;
    weightAvr[1] = b * 100;
    long Totalweight =  weightAvr[0] + weightAvr[1];
    CG= ( weightAvr[1] * WingScaleDist[type] / (Totalweight) ) + LEstopperDist[type] ;
    /*for(int x=0; x<2; x++){
      
        lcd.setCursor(8*x, 0); 
        Serial.print("weight_LdCell_");Serial.print(x);Serial.print(": ");
        
        long i = weightAvr[x];
        if (i < 0) {
          Serial.print('-');
          i = ~weightAvr[x];
        }
        lcd.print(i/100);
        lcd.print(".");
        if ((i % 100) < 10) {
          lcd.print("0");
        }
        lcd.print(i % 100);
        lcd.print("g");
        
    }*/
    Serial.print("weight_LdCell_1: ");Serial.println(a);
    Serial.print("weight_LdCell_2: ");Serial.println(b);
    Serial.println();  
    //-----------------------------------------
    lcd.setCursor(0, 0);
    if(type==0){
        lcd.print("F5D m= ");
      }
      else{
        lcd.print("F3X m= ");
      }  
    long i = Totalweight;
    if (i < 0) {
          Serial.print('-');
          i = ~Totalweight;
        }
    lcd.print(i/100);
    lcd.print(".");
    if ((i % 100) < 10) {
      lcd.print("0");
    }
    lcd.print(i % 100 / 10);
    lcd.print("g");

    lcd.setCursor(0, 1);
    lcd.print("CoG=");
    
    i = CG;
    lcd.print(i/10);
    lcd.print(".");
    lcd.print(i % 10);
    lcd.print("mm");
         
 
  }
  type_switch();
  //Serial.println(digitalRead(SwPin));
}
